library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity JKFF is
  generic (
    tco : time:=0 ps );
port (	clk:		in std_logic;
	J, K:		in std_logic;
	rst_n:		in std_logic;
	Q, Qbar:	out std_logic
);
end JKFF;

architecture behavioral of JKFF is
  signal state: std_logic;
  signal input: std_logic_vector(1 downto 0);
begin

    -- combine inputs into vector
    input <= J & K;		

    process(clk, rst_n) is
    begin
	
	if (rst_n='0') then
	    state <= '0';
	elsif (rising_edge(clk)) then

            -- compare to the truth table
	    case (input) is
		when "11" =>
		    state <= not state;
		when "10" =>
		    state <= '1';
		when "01" =>
		    state <= '0';
		when others =>
		    null;
		end case;
	end if;

    end process;
	
    -- concurrent statements
    Q <= state after tco;
    Qbar <= not state after tco;

end behavioral;
