library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.all;

entity jkff_tb is
end jkff_tb;

architecture behavior of jkff_tb is
  constant cycle_time_c : time:=166 ps;
  constant clock_to_output_c : time:=50 ps;

  -- clock_gen
  signal clk_s : std_logic;      
  signal reset_s : std_logic;

  signal J_s, K_s: 	std_logic;
  signal Q_s, Qbar_s:	std_logic;
  
begin
  
  clock_gen : entity work.clock_gen(behavioral)
    generic map (
      TCYC => cycle_time_c,
      RES_ACT_HIGH => false,
      RES_CYCLES => 1 )
    port map (
      clk   => clk_s,
      reset => reset_s );
    
  jkff : entity work.jkff(behavioral)
    generic map (
      tco => clock_to_output_c )
    port map ( clk => clk_s,
               J => J_s,
               K => K_s,
               rst_n => reset_s,
               Q => Q_s,
               Qbar => Qbar_s);
    
  testbench : process

    variable err_cnt_v : integer := 0;
    
  begin

    -- case 1
    J_s <= '0';
    K_s <= '1';
    wait for 1 ns;
    assert (Q_s='0') report "Error1!" severity error; 
    if (Q_s/='0') then
      err_cnt_v := err_cnt_v + 1;
    end if;

    -- case 2
    wait for 1 ns;
    J_s <= '1';
    K_s <= '0';
    wait for 500 ps;
    assert (Q_s='1') report "Error2!" severity error; 
    if (Q_s/='0') then
      err_cnt_v := err_cnt_v + 1;
    end if;

    -- case 3
    wait for 250 ps;
    J_s <= '1';
    K_s <= '1';
    wait for 1 ns;
    assert (Q_s='1') report "Error3!" severity error;
    if (Q_s/='1') then
      err_cnt_v := err_cnt_v + 1;
    end if;
    
    -- summary of all the tests
    if (err_cnt_v=0) then 			
      assert false 
        report "Testbench completed successfully!" 
        severity note; 
    else 
      assert true 
        report "Something wrong, try again" 
        severity error; 
    end if; 

    wait;
    
  end process testbench;
  
end behavior;
